// Console for logging debug messages
var console;
var browser;
var deck;
var mainPage
var browserVisible = false;

// Some shortcuts for accessing mozilla components
const Ci = Components.interfaces;
const Cu = Components.utils;
const Cc = Components.classes;
const nsIWebProgress = Ci.nsIWebProgress;
const nsIWebProgressListener = Ci.nsIWebProgressListener;

Cu.import("resource://gre/modules/XPCOMUtils.jsm");

var progressListener = {
    QueryInterface: function(aIID)
    {
       if (aIID.equals(Ci.nsIWebProgressListener) ||
           aIID.equals(Ci.nsISupportsWeakReference) ||
           aIID.equals(Ci.nsISupports))
           return this;
       throw Components.results.NS_NOINTERFACE;
    },

    onStateChange: function(aWebProgress, aRequest, aStateFlags, aStatus)
    {
        if (aStateFlags & nsIWebProgressListener.STATE_IS_DOCUMENT &&
            aStateFlags & nsIWebProgressListener.STATE_STOP)
        {
            if (browserVisible == false)
            {
                console.logStringMessage("Showing browser");
                deck.setAttribute("selectedIndex", "1");
                browserVisible = true;
            }
        }
    },

    onLocationChange: function(aProgress, aRequest, aURI) {},
    onProgressChange: function(aWebProgress, aRequest, curSelf, maxSelf, curTot, maxTot) {},
    onStatusChange: function(aWebProgress, aRequest, aStatus, aMessage) {},
    onSecurityChange: function(aWebProgress, aRequest, aState) {}
};

var newWindowHandler = {
    QueryInterface: XPCOMUtils.generateQI([Ci.nsIBrowserDOMWindow,
                                           Ci.nsISupports]),

    openURI: function (aURI, aOpener, aWhere, aContext) {
        console.logStringMessage(aURI + ", " + aOpener + ", " + aWhere + ", " + aContext);

        var url = aURI ? aURI.spec : "about:blank";
        var win = openDialog("chrome://application/content/main.xul",
                      "_blank", "chrome,width=200,height=200,all,dialog=no", url);
        return win;
    },

    isTabContentWindow: function (aWindow) {
        console.logStringMessage("in isTabContentWindow");
        return false;
    }
};

var appRunnerWindowCreator;
var windowCreator = {
    QueryInterface: XPCOMUtils.generateQI([Ci.nsIWindowCreator,
                                           Ci.nsIWindowCreator2,
                                           Ci.nsISupports]),

    createChromeWindow: function(parent, chromeFlags) {
        console.logStringMessage("In createChromeWindow");
    },

    createChromeWindow2: function(parent, chromeFlags, contextFlags, uri,
                                  cancel) {
        console.logStringMessage("In createChromeWindow2");
        if (uri && uri.spec && !uri.spec.match(/bingocrazy.com/))
        {
            cancel.value = true;
            return null;
        }
        return appRunnerWindowCreator.QueryInterface(Ci.nsIWindowCreator2)
                   .createChromeWindow2(parent, chromeFlags, contextFlags, uri,
                                        cancel);
    }
};

function onLoad()
{
    // Initialize console
    console = Cc["@mozilla.org/consoleservice;1"].
        getService(Ci.nsIConsoleService);

    console.logStringMessage("Initializing");

    // Initialize objects
    browser = document.getElementById("browser");
    deck = document.getElementById("deck");

    // Load configuration
    loadConfiguration();

    /*
    // Setup window open handler
    window.QueryInterface(Ci.nsIDOMChromeWindow).browserDOMWindow = newWindowHandler;

    // Register ourselves as the default window creator so we can control
    // handling of external links
    appRunnerWindowCreator = Cc["@mozilla.org/toolkit/app-startup;1"].getService(Ci.nsIWindowCreator);
    var windowWatcher = Cc["@mozilla.org/embedcomp/window-watcher;1"].getService(Ci.nsIWindowWatcher);
    windowWatcher.setWindowCreator(windowCreator);
    */

    // Add progress listener on browser
    browser.addProgressListener(progressListener, nsIWebProgress.NOTIFY_ALL |
                                nsIWebProgress.NOTIFY_STATE_ALL);

    /*
    var uri;
    // Get the URI from arguments in case this is a new window
    if ("arguments" in window && window.arguments[0] &&
        typeof(window.arguments[0]) == "string")
        uri = window.arguments[0];
    else
        uri = mainPage;

    console.logStringMessage ("Loading URI: " + uri);
    */

    //window.location = mainPage;
    browser.webNavigation.loadURI(mainPage, Ci.nsIWebNavigation.LOAD_FLAGS_NONE, null, null, null);

    // Check for updates immediately on startup
    // checkForUpdates();
}

function checkForUpdates()
{
    var updateChecker = Cc["@mozilla.org/updates/update-checker;1"].
        getService(Ci.nsIUpdateChecker);

    updateChecker.checkForUpdates(updateCheckListener, false);
}

var updateCheckListener = {
    QueryInterface: XPCOMUtils.generateQI([Ci.nsIUpdateCheckListener,
                                           Ci.nsISupports]),

    onCheckComplete: function (request, updates, updateCount) {
        console.logStringMessage("Update checking successful: " + updates.length);

        var aus = Cc["@mozilla.org/updates/update-service;1"].
            getService(Ci.nsIApplicationUpdateService);

        var update = aus.selectUpdate(updates, updates.length);
        if (!update) {
            console.logStringMessage("No updates found");
            return;
        }

        aus.downloadUpdate(update, true);
    },

    onError: function (request, update) {
        console.logStringMessage("Update checking failed");
    },

    onProgress: function (request, position, totalSize) {}
};

function loadConfiguration()
{
    // Get the "application.mainPage" preference
    var prefs = Cc["@mozilla.org/preferences-service;1"]
                    .getService(Ci.nsIPrefService).getBranch("application.");
    mainPage = prefs.getCharPref("mainPage");
    mainPage = mainPage.replace(/%INSTALL_ID%/, getInstallID());
}

function getInstallID()
{
    try {
        var id;
        var key = Cc["@mozilla.org/windows-registry-key;1"]
                      .createInstance(Ci.nsIWindowsRegKey);
        key.open(key.ROOT_KEY_LOCAL_MACHINE,
            "SOFTWARE", key.ACCESS_READ);
        if (key.hasChild("Bingo Crazy")) {
          var subkey = key.openChild("Bingo Crazy", key.ACCESS_READ);
          id = subkey.readStringValue("InstallID");
          subkey.close();
        }
        else {
          key.close();
          key.open(key.ROOT_KEY_CURRENT_USER,
            "SOFTWARE\\Bingo Crazy", key.ACCESS_READ);
          id = key.readStringValue("InstallID");
        }
        key.close();

        return id;
    } catch (ex) {
        return "0";
    }
}

window.addEventListener ("load", onLoad, false);
